/*
 * This is a sample code to use BCSearch, a fast similarity scoring for 
 * ungapped fragments.
 *
 * This program is to search for similar fragments to a seed specified 
 * in input in a query protein. 
 * Input format is xyz, one line par alpha carbon.
 * Matches with a score more than a given BC score thresholds are returned.
 *
 * @Authors: F. Guyon, P. Tuffery , 2011-2013 
 * For comments, suggestions, please contact 
 * frederic.guyon@univ-paris-diderot.fr or pierre.tuffery@univ-paris-diderot.fr
 * 
 * This program CANNOT presently be used, modified and distributed without the
 * explicit agreement of the authors, F. Guyon and P. Tuffery
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "argstr.h"
#include "util.h"

/*
 * det(K): determinant de matrice 3x3
 */
double det3x3(double K[3][3]) {
  return K[0][0]*(K[1][1]*K[2][2]-K[2][1]*K[1][2])+K[1][0]*(K[2][1]*K[0][2]-K[0][1]*K[2][2])+K[2][0]*(K[0][1]*K[1][2]-K[1][1]*K[0][2]);
}

double mirror (double *X, double *Y, int n) {
  // Xki=X[k+i*3]
  double K[3][3],Xm[3], Ym[3];
  double det, sum;
  int i,j,k;

  for (i=0; i<3;i++) {
    sum=0;
    for (j=0; j<n;j++)
      sum+=X[i+j*3]; 
    Xm[i]=sum/n;
  }
  for (i=0; i<3;i++) {
    sum=0;
    for (j=0; j<n;j++)
      sum+=Y[i+j*3]; 
    Ym[i]=sum/n;
  }

  for (i=0; i<3;i++)
    for (j=0; j<3; j++) {
      sum=0;
      for (k=0; k<n;k++)
	sum+=(Y[i+3*k]-Ym[i])*(X[j+3*k]-Xm[j]);
      K[i][j]=sum;
    }
  det=det3x3(K);
  return det;
}

/*
 *  score gBC (global Binet-Cauchy)
 */ 
double globdet (double *X, double *Y,int n) {
  double det, detX, detY;

   det=mirror (X, Y, n);
   detX=mirror (X, X, n);
   detY=mirror(Y, Y, n);
  return det/sqrt(detX*detY);
  return det;
}


void locdet_slide(double *X,  int N, double *Y, int M, int flen, double *results) {
  int i;
  double  r;
  
  for (i=0; i<N-flen+1; i++) {
    r=globdet(&(X[3*i]), &(Y[3*i]), flen);
    results[i]=r;
  }
}

/*
 * MAIN
 */
int main(int argc, char *argv[]) {
  FILE *logf;
  DoubleVector *X1=NULL, *X2=NULL;
  int len1, len2;
  FILE *file1, *file2, *file3;
  int status;
  char id1[MAXLINE], id2[MAXLINE], filename[MAXLINE], ofilename[MAXLINE], header[MAXID];
  double gbc;
  int nresult;
  int nOK, i;

  /*
   * lecture des arguments 
   */
  parseArgstr(argc, argv);
  if ((IFragFichName[0] == '\000') || (IProtFichName[0] == '\000')) {
    fprintf (stderr, "Sorry: must specify input data\n");
    return -1;
  }
  /*
   * quick check files
   */
  if ((file1=fopen(IFragFichName, "r"))==NULL) {
    fprintf (stderr, "Sorry: cannot open xyz file for query: %s\n", IFragFichName);
    return -1;
  }

  if (!strcmp(OLogFichName,"stderr")) logf = stderr;
  else if (!strcmp(OLogFichName,"stdout")) logf = stdout;
  else logf = fopen(OLogFichName,"w");
  prmtrsSumary(logf);

  /*
   * lecture du fragment query
   */
  if (gVerbose) {
    fprintf (stderr, "Reading %s\n", IFragFichName);
  }
  X1=readDoubleVector (file1, X1, &status, id1, gPDBxyz);
  len1=X1->size/3;
  fclose(file1);
  if (gVerbose) {
    fprintf (stderr, "  Read %d coordinates\n", len1);
  }

  /*
   * main loop 
   */
  // printf("%13s %10s %5s %5s %5s %5s %7s %6s %9s %5s\n", "File1","File2","from1","to1","from2","to2","rmsd","BC","Pvalue","distortion");

  strcpy(filename, IProtFichName);
  if ((file3=fopen(filename, "r"))==NULL) {
    fprintf (stderr, "Sorry: cannot open protein file to scan: %s\n", IProtFichName);
    exit(-1);
  }
  if (gVerbose) {
    fprintf (stderr, "Reading %s\n", filename);
  }

  strcpy(ofilename, filename);
  X2=readDoubleVector (file3, X2, &status, id2, gPDBxyz);
  //fprintf(stderr, "id2=%s\n",id2);
  len2=X2->size/3;
  if (gVerbose) {
    fprintf (stderr, "  Read %d coordinates\n", len2);
  }

  /* Check that lengths are compatible */
  if (len2 != len1) { 
    if (gVerbose) {
      fprintf(stderr, "Input data of different size %d %d \n", len1, len2);
    }
    fclose(file3);
    freeDoubleVector(X1);
    freeDoubleVector(X2);
    exit(0);
  }

  /* Global */
  gbc = globdet (X1->values, X2->values, len1);
  nresult = len1-gFLen+1;

  printf("%13s %13s %lf\n", IFragFichName, IProtFichName, gbc);


  fclose(file3);
  freeDoubleVector(X2);
  
  freeDoubleVector(X1);
}
